#Please install zeep and xmltodict libs using these commands in Command window first:
#pip install zeep
#pip install xmltodict

from zeep import Client
import xmltodict
from datetime import datetime

# Please update these lines with your own companyname, userame, and password
username = "YOUR_USERNAME"
password = "YOUR_PASSWORD"
companyName = "YOUR_COMPANY_NAME"
base_url = 'https://www.methodintegration.com/'


table_name = 'Customer'
unique = datetime.now().strftime('%m%d%H%M%S')
insert_fields = {
        "Name": "C-"+unique,
        "CompanyName": "Customer-A1",
        "FirstName": "John",
        "LastName": "Smith",
        "AssignedTo": "QA Team",
        "Currency": "USD",
        "BillAddressAddr1": "UNSW Global",
        "BillAddressAddr2": "12-22",
        "BillAddressAddr3": "Rothschild Avenue",
        "BillAddressAddr4": "PO Box 8020",
        "BillAddressAddr5": "Assessment centre",
        "BillAddressCity": "Alexandria",
        "BillAddressCountry": "Australia",
        "BillAddressPostalCode": "1435",
        "BillAddressState": "NSW",
        "ShipAddressAddr1": "Herrn",
        "ShipAddressAddr2": "Eberhard Wellhausen",
        "ShipAddressAddr3": "Wittekindshof",
        "ShipAddressAddr4": "Schulstrasse 4",
        "ShipAddressAddr5": "Towards South",
        "ShipAddressCity": "Bad Oyenhausen",
        "ShipAddressCountry": "Germany",
        "ShipAddressPostalCode": "32547",
        "ShipAddressState": "NA",
        "AltPhone": "(234)234-2345",
        "Email": "xyz@abc.com",
        "Fax": "(234)234-2346",
        "Terms": "Net 30"}

#Initializing the zeep
client = Client(wsdl=base_url+'MethodAPI/service.asmx?WSDL')
factory = client.type_factory('ns0')

# There is a bit of weiredness with passing arrays to zeep and it needs to be converted like this:
names = factory.ArrayOfString(list(insert_fields.keys()))
values = factory.ArrayOfString(list(insert_fields.values()))

# calling our Insert endpoint:
xml_str = client.service.MethodAPIInsertV2(
    companyName,
    username, password, None,
    table_name, names, values)
assert xml_str is not None
insert_result = xmltodict.parse(xml_str)
assert 'Success' in insert_result['MethodAPI']['@response'], xml_str
assert '@RecordID' in insert_result['MethodAPI']
assert insert_result['MethodAPI']['@RecordID'] is not None
record_id = insert_result['MethodAPI']['@RecordID']
print('Insert API completed: RecordId='+record_id)

# calling our Select endpoint
xml_str = client.service.MethodAPISelect_XMLV2(
    companyName,
    username, password, None,
    table_name, ','.join(list(insert_fields.keys())), 'RecordID=' + record_id)
assert xml_str is not None
select_result = xmltodict.parse(xml_str)
assert 'Success' in select_result['MethodAPI']['@response'], xml_str
assert 'Record' in select_result['MethodAPI']['MethodIntegration']
row = select_result['MethodAPI']['MethodIntegration']['Record']
print('Retrieved values from Select API:')
print(row)

update_fields = {
    "CompanyName": "Customer-B1",
    "FirstName": "Jane"
}
names = factory.ArrayOfString(list(update_fields.keys()))
values = factory.ArrayOfString(list(update_fields.values()))

# calling our Update endpoint:
xml_str = client.service.MethodAPIUpdateV2(
    companyName,
    username, password, None,
    table_name, names, values, record_id)
assert xml_str is not None
update_result = xmltodict.parse(xml_str)
assert 'Success' in update_result['MethodAPI']['@response'], xml_str
print('Update API completed')


# calling our Delete endpoint:
xml_str = client.service.MethodAPIDeleteV2(
    companyName,
    username, password, None, 'Customer', record_id)
assert xml_str is not None
result_dict = xmltodict.parse(xml_str)
assert 'Success' in result_dict['MethodAPI']['@response'], xml_str
print('Delete API completed')

